@@ -12,7 +12,7 @@ use File::Spec     ();
 use constant DEBUG    => $ENV{MOJO_ASSETPACK_DEBUG}    || 0;
 use constant NO_CACHE => $ENV{MOJO_ASSETPACK_NO_CACHE} || 0;
 
-our $VERSION = '0.63';
+our $VERSION = '0.64';
 
 has base_url      => '/packed/';
 has headers       => sub { +{} };
@@ -36,10 +36,10 @@ sub add {
 }
 
 sub fetch {
-  my $self = shift;
-  my $url  = Mojo::URL->new(shift);
-  my $asset
-    = $self->_packed("$url") || $self->_handler($url->scheme)->asset_for($url, $self)->in_memory(!$self->out_dir)->save;
+  my $self  = shift;
+  my $url   = Mojo::URL->new(shift);
+  my $asset = $self->_packed("$url")
+    || $self->_handler($url->scheme)->asset_for($url, $self)->in_memory($self->out_dir ? 0 : 1)->save;
   return $asset if @_;    # internal
   return $asset->path;    # documented api
 }
@@ -95,6 +95,9 @@ sub register {
   $self->{processed}    = {};
   $self->{source_paths} = $self->_build_source_paths($app, $config);
 
+  # Not official. Want to keep it private for now...
+  $self->{die_on_process_error} = $ENV{MOJO_ASSETPACK_DIE_ON_PROCESS_ERROR} // $app->mode ne 'development';
+
   $self->_app($app);
   $self->_ua->server->app($app);
   $self->_ua->proxy->detect if $config->{proxy};
@@ -107,7 +110,9 @@ sub register {
   if (NO_CACHE) {
     $app->log->info('AssetPack Will rebuild assets on each request in memory');
     $self->out_dir('');
-    $self->_assets_from_memory;
+  }
+  if (!$self->out_dir) {
+    $self->{assets_from_memory} = 1;
   }
 
   $app->helper(
@@ -117,6 +122,8 @@ sub register {
       return $self->_inject(@_);
     }
   );
+
+  Mojo::IOLoop->next_tick(sub { $self->_add_hook($app, $config) });
 }
 
 sub source_paths {
@@ -132,6 +139,40 @@ sub source_paths {
   }
 }
 
+sub _add_hook {
+  my ($self, $app, $config) = @_;
+  my $headers = $self->headers;
+
+  if ($self->{assets_from_memory}) {
+    $app->hook(
+      before_routes => sub {
+        my $c    = shift;
+        my $path = $c->req->url->path;
+
+        return if $c->req->is_handshake or $c->res->code;
+        return unless $path->[1] and 0 == index "$path", $self->base_url;
+        return unless my $asset = $c->asset->{asset}{$path->[1]};
+        return if $asset->{internal};
+        my $h = $c->res->headers->last_modified(Mojo::Date->new($^T))
+          ->content_type($c->app->types->type($asset->path =~ /\.(\w+)$/ ? $1 : 'txt') || 'text/plain');
+        $h->header($_ => $headers->{$_}) for keys %$headers;
+        $c->reply->asset($asset);
+      }
+    );
+  }
+  elsif (%$headers) {
+    $app->hook(
+      after_static => sub {
+        my $c    = shift;
+        my $path = $c->req->url->path;
+        return unless $path->[1] and 0 == index "$path", $self->base_url;
+        my $h = $c->res->headers;
+        $h->header($_ => $headers->{$_}) for keys %$headers;
+      }
+    );
+  }
+}
+
 sub _asset {
   my ($self, $name) = @_;
   my $asset = $self->{asset}{$name} ||= Mojolicious::Plugin::AssetPack::Asset->new;
@@ -145,27 +186,6 @@ sub _assets {
   $self->{assets}{$moniker} || [];
 }
 
-sub _assets_from_memory {
-  my $self    = shift;
-  my $headers = $self->headers;
-
-  $self->_app->hook(
-    before_routes => sub {
-      my $c    = shift;
-      my $path = $c->req->url->path;
-
-      return if $c->req->is_handshake or $c->res->code;
-      return unless $path->[1] and 0 == index "$path", $self->base_url;
-      return unless my $asset = $c->asset->{asset}{$path->[1]};
-      return if $asset->{internal};
-      my $h = $c->res->headers->last_modified(Mojo::Date->new($^T))
-        ->content_type($c->app->types->type($asset->path =~ /\.(\w+)$/ ? $1 : 'txt') || 'text/plain');
-      $h->header($_ => $headers->{$_}) for keys %$headers;
-      $c->reply->asset($asset);
-    }
-  );
-}
-
 sub _build_out_dir {
   my ($self, $app, $config) = @_;
   my $out_dir = $config->{out_dir};
@@ -235,7 +255,7 @@ sub _inject {
 
   eval {
     if ($args->{inline}) {
-      return $c->$tag_helper(@attrs, sub { join "\n", $self->get($moniker, $args) });
+      return $c->$tag_helper(@attrs, sub { join '', $self->get($moniker, $args) });
     }
     else {
       return Mojo::ByteStream->new(join "\n", map { $c->$tag_helper($_, @attrs) } $self->get($moniker, $args));
@@ -288,59 +308,57 @@ sub _packed {
 
 sub _process {
   my ($self, $moniker, @sources) = @_;
+  my $app   = $self->_app;
+  my $topic = $moniker;
   my ($name, $ext) = (_name($moniker), _ext($moniker));
   my ($asset, $file, @checksum);
 
-  @sources = map {
-    my $source = $self->_source_for_url($_);
-    push @checksum, $self->preprocessors->checksum(_ext($_), \$source->slurp, $source->path);
-    $source;
-  } @sources;
+  eval {
+    for my $s (@sources) {
+      $topic = $s;
+      $s     = $self->_source_for_url($s);    # rewrite @sources
+      push @checksum, $self->preprocessors->checksum(_ext($topic), \$s->slurp, $s->path);
+    }
 
-  @checksum = (Mojo::Util::md5_sum(join '', @checksum)) if @checksum > 1;
-  $asset = $self->_packed($self->minify ? qr{^$name-$checksum[0](\.min)?\.$ext$} : qr{^$name-$checksum[0]\.$ext$});
-  return $asset if $asset;
+    @checksum = (Mojo::Util::md5_sum(join '', @checksum)) if @checksum > 1;
+    $asset = $self->_packed($self->minify ? qr{^$name-$checksum[0](\.min)?\.$ext$} : qr{^$name-$checksum[0]\.$ext$});
+    return $asset if $asset;                  # already processed
 
-  $file = $self->minify ? "$name-$checksum[0].min.$ext" : "$name-$checksum[0].$ext";
-  $asset = $self->{asset}{$file} = Mojolicious::Plugin::AssetPack::Asset->new;
-  $asset->in_memory(1)->path(File::Spec->catfile($self->out_dir, $file));
+    $file = $self->minify ? "$name-$checksum[0].min.$ext" : "$name-$checksum[0].$ext";
+    $asset = $self->{asset}{$file} = Mojolicious::Plugin::AssetPack::Asset->new;
+    $asset->in_memory(1)->path(File::Spec->catfile($self->out_dir, $file));
 
-  for my $source (@sources) {
-    eval {
-      my $content = $source->slurp;
-      $self->preprocessors->process(_ext($source->path), $self, \$content, $source->path);
+    for my $s (@sources) {
+      $topic = $s->basename;
+      my $content = $s->slurp;
+      $self->preprocessors->process(_ext($s->path), $self, \$content, $s->path);
       $asset->content($asset->content . $content);
-      1;
-    } or do {
-      my $e = $@;
-      warn "[ASSETPACK] process(@{[$source->path]}) FAIL $e\n" if DEBUG;
-      $asset->path(File::Spec->catfile($self->out_dir, "$name-$checksum[0].err.$ext"));
-      $asset->content($self->_make_error_asset($moniker, $source->basename, $e || 'Unknown error'));
-      last;
-    };
+    }
+
+    $asset->in_memory($self->out_dir ? 0 : 1)->save;
+  } or do {
+    my $err          = $@;
+    my $source_paths = join ',', @{$self->source_paths};
+    my $static_paths = join ',', @{$app->static->paths};
+    $err =~ s!\s+$!!;    # remove newlines
+    my $msg = "AssetPack failed to process $topic: $err {source_paths=[$source_paths], static_paths=[$static_paths]}";
+    $app->log->error($msg);
+    die $msg if $self->{die_on_process_error};    # prevent hot reloading when assetpack fail
+    $file ||= "$name-@{[time]}.$ext";
+    $file =~ s!$ext$!err.$ext!;
+    $asset = $self->{asset}{$file} = Mojolicious::Plugin::AssetPack::Asset->new(in_memory => 1, path => $file);
+    $asset->content($self->_make_error_asset($moniker, $topic, $err || 'Unknown error'));
+  };
+
+  if ($asset->in_memory) {
+    $app->log->info("AssetPack built @{[$asset->path]} for @{[$app->moniker]} in memory.");
+    $self->{assets_from_memory}++;
+  }
+  elsif ($file) {
+    $app->log->info("AssetPack built @{[$asset->path]} for @{[$app->moniker]}.");
   }
 
-  unless ($self->{hook_added}++) {
-    if (!$self->out_dir) {
-      $self->_app->log->warn('AssetPack will store assets in memory');
-      $self->_assets_from_memory;
-    }
-    elsif (my $headers = $self->headers) {
-      $self->_app->hook(
-        after_static => sub {
-          my $c    = shift;
-          my $path = $c->req->url->path;
-          return unless $path->[1] and 0 == index "$path", $self->base_url;
-          my $h = $c->res->headers;
-          $h->header($_ => $headers->{$_}) for keys %$headers;
-        }
-      );
-    }
-  }
-
-  $asset->in_memory(!$self->out_dir)->save;
-  $self->_app->log->info("Built asset for $moniker");
-  $asset;
+  return $asset;
 }
 
 sub _process_many {
@@ -378,7 +396,7 @@ sub _source_for_url {
   my @path = split '/', $url;
 
   for my $path (map { Cwd::abs_path(File::Spec->catfile($_, @path)) } @look_in) {
-    return $self->_asset($_)->path($path)->in_memory(0) if $path and -r $path;
+    return $self->_asset($path)->path($path)->in_memory(0) if $path and -r $path;
   }
 
   return $self->_handler('https')->asset_for($url, $self);
@@ -409,7 +427,7 @@ Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass, javascrip
 
 =head1 VERSION
 
-0.63
+0.64
 
 =head1 SYNOPSIS
 
@@ -533,8 +551,8 @@ NOTE! You need to have a trailing "/" at the end of the string.
 
 =head2 headers
 
+  $app->plugin("AssetPack" => {headers => {"Cache-Control" => "max-age=31536000"}});
   $hash_ref = $self->headers;
-  $self = $self->headers({"Cache-Control" => "max-age=31536000"});
 
 This attribute can hold custom response headers when serving assets.
 The default is no headers, but this might change in future release to
@@ -544,8 +562,8 @@ This attribute is EXPERIMENTAL.
 
 =head2 minify
 
-  $bool = $self->minify;
   $app->plugin("AssetPack" => {minify => $bool});
+  $bool = $self->minify;
 
 Set this to true if the assets should be minified.
 
@@ -555,8 +573,8 @@ See also L<Mojolicious::Plugin::AssetPack::Manual::Modes>.
 
 =head2 out_dir
 
-  $str = $self->out_dir;
   $app->plugin("AssetPack" => {out_dir => $str});
+  $str = $self->out_dir;
 
 Holds the path to the directory where packed files can be written.
 
